##quantitative genetics simulations
##simulate phenotypes, then estimate narrow sense heritability & response to selection
##original code by Graham Coop
##modified by Nancy Chen
##April 2017

##function to simulate trios and calculate narrow sense heritability for traits with different environmental variances (environ.var)
par.off.corr<-function(L=20, environ.var, Num_inds=1000){
	##each locus is assumed to have the same allele frequencies. This is just to simplify the coding, in reality these results work when each locus has its own frequency (and the coding wouldn't be too much harder). 
	allele.freq<-0.5   
		 
	##simulate trios
	##MAKE A MUM
	##For each mother, at each locus we draw an allele (either 0 or 1) from the population allele frequency. 
	##We do this twice for each mother two represent the two haplotypes in the mother 
	mum.hap.1<-replicate(Num_inds, rbinom(L, 1, allele.freq))
	mum.hap.2<-replicate(Num_inds, rbinom(L, 1, allele.freq))
	##type mum.hap.1[, 1] to see the 1st mothers 1st haplotype
	
	##Each mothers genotype at each locus is either 0, 1, 2
	mum.geno<-mum.hap.1+mum.hap.2
	
	##normalize this to have variance 1, to make it comparable to the environmental variance.
	additive.genetic<-colSums(mum.geno)
	genetic.sd<-sd(additive.genetic)
	mean.genetic<-mean(additive.genetic)
	
	additive.genetic<-additive.genetic / sd(additive.genetic)
	##add the environmental contribution to the trait to get the overall phenotype
	mum.pheno<-additive.genetic + rnorm(Num_inds, sd=sqrt(environ.var))
	##normalize the trait to have mean zero
	mum.pheno<-mum.pheno-mean(mum.pheno)
	
	##MAKE A DAD (same code as make a mum)
	dad.hap.1<-replicate(Num_inds, rbinom(L, 1, allele.freq))
	dad.hap.2<-replicate(Num_inds, rbinom(L, 1, allele.freq))
	dad.geno<-dad.hap.1+dad.hap.2
	
	additive.genetic<-colSums(dad.geno)
	additive.genetic<-additive.genetic / sd(additive.genetic)
	dad.pheno<-additive.genetic + rnorm(Num_inds, sd=sqrt(environ.var))
	dad.pheno<-dad.pheno-mean(dad.pheno)
	
	##MAKE A CHILD
	child.geno<-dad.hap.1+mum.hap.1 ##1 haplotype from mum 1 haplotype from dad
	
	additive.genetic<-colSums(child.geno)
	additive.genetic<-additive.genetic / sd(additive.genetic)
	child.pheno<-additive.genetic + rnorm(Num_inds, sd=sqrt(environ.var))
	child.pheno<-child.pheno-mean(child.pheno)
	
	##Estimate narrow sense heritability using midparent-offspring regression
	##avg. parents
	parental.midpoint<-(mum.pheno+dad.pheno)/2 	
	
	##linear model between child and mid point
	##the slope of this line is the narrow sense heritability.
	lm(child.pheno~parental.midpoint) 
	my.slope<-lm(child.pheno~parental.midpoint)$coeff[2]
	
	##plot parental midpoint against offsprings phenotype.
	#layout(1) ###done in case this is run after the code with 3 plots
	plot(parental.midpoint, child.pheno, xlab="Parental midpoint", ylab="Child's phenotype", cex=1.5, cex.axis=1.5, cex.main=1.5, cex.lab=1.5, main=bquote("L =" ~ .(L) ~ ", VE =" ~ .(environ.var) ~ ", " ~ h^{2} ~ "=" ~ .(format(my.slope, digit=3))))
	
	##plot the regression in blue
	abline(h=0, col="grey", lwd=2)
	abline(v=0, col="grey", lwd=2)
	abline(lm(child.pheno~parental.midpoint), col="#56B4E9", lwd=3)
		
	abline(0, 1, col="#E69F00", lwd=3, lty=2)
 #	recover()
 }
 
##Phenotypic variance is a function of additive genetic variance and environmental variance 
##We can estimate narrow sense heritability using midparent-offspring regressions (better methods exist but intuition is the same)
##TODO: Try different values of environ.var. What happens as the environmental variance increases?
layout(t(1:3))
par.off.corr(L=100, environ.var=100, Num_inds=500)
par.off.corr(L=100, environ.var=1, Num_inds=500)
par.off.corr(L=100, environ.var=0.001, Num_inds=500)

	

##function to simulate trios and response to selection for traits with different environmental variances (environ.var) and selection differentials (sel)
one.gen.sel<-function(L=1000, environ.var, sel){
	##each locus is assumed to have the same allele frequencies. This is just to simplify the coding, in reality these results work when each locus has its own frequency (and the coding wouldn't be too much harder). 
	allele.freq<-0.5   
	 
	#number of individuals
	Num_inds=10000
	
	##simulate trios (same as above)
	##MAKE A MUM
	##For each mother, at each locus we draw an allele (either 0 or 1) from the population allele frequency. 
	##We do this twice for each mother two represent the two haplotypes in the mother 
	mum.hap.1<-replicate(Num_inds, rbinom(L, 1, allele.freq))
	mum.hap.2<-replicate(Num_inds, rbinom(L, 1, allele.freq))
	##type mum.hap.1[, 1] to see the 1st mothers 1st haplotype
	
	##Each mothers genotype at each locus is either 0, 1, 2
	mum.geno<-mum.hap.1+mum.hap.2
	
	additive.genetic<-colSums(mum.geno)
	mean.genetic<-mean(additive.genetic)
	genetic.var<-sd(additive.genetic)
	
	additive.genetic<-additive.genetic / sd(additive.genetic)
	mum.pheno<-additive.genetic + rnorm(Num_inds, sd=sqrt(environ.var))
	mum.pheno<-mum.pheno-mean(mum.pheno)
	
	##MAKE A DAD (same code as make a mum)
	dad.hap.1<-replicate(Num_inds, rbinom(L, 1, allele.freq))
	dad.hap.2<-replicate(Num_inds, rbinom(L, 1, allele.freq))
	dad.geno<-dad.hap.1+dad.hap.2
	
	additive.genetic<-colSums(dad.geno)
	additive.genetic<-additive.genetic / sd(additive.genetic)
	dad.pheno<-additive.genetic + rnorm(Num_inds, sd=sqrt(environ.var))
	dad.pheno<-dad.pheno-mean(dad.pheno)
	
	##MAKE A CHILD
	##Selection of top sel% of individuals	 
	top.sel.per.mums<-mum.pheno>quantile(mum.pheno, p=1-sel) 
	top.sel.per.dads<-dad.pheno>quantile(dad.pheno, p=1-sel)
	 
	child.geno<-dad.hap.1[, top.sel.per.dads]+mum.hap.1[, top.sel.per.mums] ##1/2 from mum 1/2 from dad
	
	additive.genetic<-(colSums(child.geno)-mean.genetic)
	additive.genetic<-additive.genetic/genetic.var
	child.pheno<-additive.genetic + rnorm(length(child.geno), sd=sqrt(environ.var))
	
	##plot results
	layout(matrix(c(1:5, 0), 3, 2))
	
	#first plot phenotypic distributions
	my.lim<-quantile(c(mum.pheno, dad.pheno), p=c(0.01, 0.99))
	my.lim[2]<-quantile(child.pheno, p=c(0.99))
	
	hist(c(mum.pheno, dad.pheno), breaks=100, xlim=my.lim, xlab="Phenotype", main=paste("Phenotype distribution before selection, mean=0, VE=", environ.var, ", Sel top ", round(100*sel), "%", sep=""), cex.axis=1.5, cex.lab=1.5, cex.main=1.2); 
	abline(v=0, col="black", lwd=3)
	
	sel.par.mean<-mean(c(mum.pheno[top.sel.per.mums], dad.pheno[top.sel.per.dads]))
	hist(c(mum.pheno[top.sel.per.mums], dad.pheno[top.sel.per.dads]), breaks=100, xlim=my.lim, xlab="Phenotype", main=paste("Phenotype distribution after selection, mean=", format(sel.par.mean, dig=4)), cex.axis=1.5, cex.lab=1.5, cex.main=1.2); 
	abline(v=sel.par.mean, col="blue", lwd=3)
	abline(v=0, col="black", lwd=3)
	text(x=sel.par.mean/2,y=60,"S",cex=1.5,col="blue")
	arrows(x0=0,x1=sel.par.mean,y0=50,y1=50,col="blue",lwd=2,length=0.1,code=3)
	
	hist(child.pheno, xlim=my.lim, breaks=100, xlab="Phenotype", main=paste("Phenotype distribution in the children, mean=", format(mean(child.pheno), dig=4)), cex.axis=1.5, cex.lab=1.5, cex.main=1.2); 
	abline(v=0, col="black", lwd=3)
	abline(v=mean(child.pheno), col="blue", lwd=3)
	text(x=mean(child.pheno)/2,y=250,"R",cex=1.5,col="blue")
	arrows(x0=0,x1=mean(child.pheno),y0=200,y1=200,col="blue",lwd=2,length=0.1,code=3)
	
	##Mean phenotype after selection
	cat("Selected parental mean", sel.par.mean, "\n")
	##Mean child phenotype
	cat("Mean in children=", mean(child.pheno), "\n")

	#plot histogram of parent and child allele frequencies
	#hist of parents
	sel.dad.genosum<-colSums(dad.geno[, top.sel.per.dads])
	rand.dad.genosum<-colSums(dad.geno[, sample(top.sel.per.dads)])
	hist(sel.dad.genosum, breaks=20, col=rgb(1, 0, 0, 0.4), xlim=c(min(rand.dad.genosum)-5, max(sel.dad.genosum)+5), xlab="Num. tall alleles", main="Parental generation", cex.lab=1.5)
	hist(rand.dad.genosum, breaks=20, col=rgb(0, 0, 1, 0.4), add=TRUE)
	legend ("topleft", legend=c("All individuals", "Selected parents"), pch=15, col=c(rgb(0, 0, 1, 0.4), rgb(1, 0, 0, 0.4)), bty="n", cex=1.2)
	#hist of kids
	hist(colSums(child.geno), breaks=20,  col =rgb(0, 1, 0, 0.4), xlim=c(min(rand.dad.genosum)-5, max(sel.dad.genosum)+5), xlab="Num. tall alleles", main="Next generation", cex.lab=1.5)
	legend("topleft", legend=c("Children"), pch=15, col=rgb(0, 1, 0, 0.4), bty="n", cex=1.2)
}

##The response to selection (R) is a function of (realized) narrow sense heritability and the selection gradient (S)
##Breeder's equation: R = h^2*S
##TODO: Simulate phenotypic and genotypic distributions in parents and offspring before and after 1 generation of selection 
##TODO: Try different values of environ.var (0.01, 1, 50) and sel (0.1 - 0.4)
one.gen.sel(L=100, environ.var=0.01, sel=0.1)
one.gen.sel(L=100, environ.var=1, sel=0.1)
one.gen.sel(L=100, environ.var=50, sel=0.1)

one.gen.sel(L=100, environ.var=1, sel=0.1)
one.gen.sel(L=100, environ.var=1, sel=0.2)
one.gen.sel(L=100, environ.var=1, sel=0.3)
one.gen.sel(L=100, environ.var=1, sel=0.4)


